% Autogenerated with DRAKON Editor 1.11

-module(parser).
-export([calculate/1, lex/1, parse/1]).
-record(context, {operations, variables, functions}).

add_token(Buffer, Tokens) ->
    % item 222
    {_BufferType, BufferText} = Buffer,
    % item 218
    if BufferText =:= [] -> 
        % item 219
        Tokens
    ; true ->
        % item 217
        [Buffer | Tokens]
    end
.

calc_function(Tree, Context) ->
    % item 715
    [function, Name, Operand ] = Tree,
    % item 713
    Functions = Context#context.functions,
    % item 716
    Exists = dict:find(Name, Functions),
    % item 717
    if Exists =:= error -> 
        % item 720
        throw("Function " ++ Name ++ " not found")
    ; true ->
        % item 721
        OperandValue = calculate_kernel(Operand, Context),
        % item 722
        {ok, Function} = Exists,
        % item 723
        Function(OperandValue)
    end
.

calc_id(Tree, Context) ->
    % item 639
    Text = lists:nth(2, Tree),
    % item 660
    Variables = Context#context.variables,
    % item 646
    Exists = dict:find(Text, Variables),
    % item 647
    if Exists =:= error -> 
        % item 644
        throw("Variable " ++ Text ++ " not found")
    ; true ->
        % item 640
        {ok, Value} = Exists,
        % item 645
        Value
    end
.

calc_mod(Tree, Context) ->
    % item 703
    [mod, First, Second] = Tree,
    % item 705
    FirstOperand = calculate_kernel(First, Context),
    % item 706
    SecondOperand = calculate_kernel(Second, Context),
    % item 707
    Result = trunc(FirstOperand) rem trunc(SecondOperand),
    % item 708
    Result
.

calc_number(Tree, _Context) ->
    % item 627
    Text = lists:nth(2, Tree),
    % item 754
    Pos = string:chr(Text, $.),
    % item 750
    if Pos =:= 0 -> 
        % item 752
        {Value, Message} = string:to_integer(Text)
    ; true ->
        % item 628
        {Value, Message} = string:to_float(Text)
    end,
    % item 629
    if Value =:= error -> 
        % item 632
        throw(Message)
    ; true ->
        % item 633
        Value
    end
.

calc_prod(Tree, Context) ->
    % item 652
    [prod, First | Rest] = Tree,
    % item 671
    {_, FirstOperand} = First,
    % item 672
    FirstValue = calculate_kernel(FirstOperand, Context),
    % item 674
    Acc = {FirstValue, Context},
    % item 673
    NewAcc = lists:foldl(fun prod_item/2, Acc, Rest),
    % item 756
    element(1, NewAcc)
.

calc_sum(Tree, Context) ->
    % item 693
    [sum | Operands] = Tree,
    % item 698
    Acc = {0, Context},
    % item 697
    NewAcc = lists:foldl(fun sum_item/2, Acc, Operands),
    % item 755
    element(1, NewAcc)
.

calculate(Text) ->
    % item 597
    {ParseSuccess, Tree} = parse(Text),
    % item 603
    if ParseSuccess =:= ok -> 
        % item 598
        if Tree =:= [] -> 
            % item 602
            {ok, 0}
        ; true ->
            % item 612
            Variables = make_variables(),
            Functions = make_functions(),
            Operations = make_operations(),
            % item 659
            Context = #context{
            	operations = Operations,
            	variables  = Variables,
            	functions  = Functions},
            % item 604
            try
            	Value = calculate_kernel(Tree, Context),
            	{ok, Value}
            catch
            	Message ->
            		{error, Message}
            end
        end
    ; true ->
        % item 606
        {error, Tree}
    end
.

calculate_kernel(Tree, Context) ->
    % item 730
    Type = hd(Tree),
    % item 732
    Operations = Context#context.operations,
    % item 729
    Operation = dict:fetch(Type, Operations),
    % item 731
    Operation(Tree, Context)
.

is_char(Char) ->
    % item 90
    if ((Char =:= $_) orelse ((Char >= $A) andalso (Char =< $Z))) orelse ((Char >= $a) andalso (Char =< $z)) -> 
        % item 94
        true
    ; true ->
        % item 106
        false
    end
.

is_digit(Char) ->
    % item 114
    if (Char >= $0) andalso (Char =< $9) -> 
        % item 117
        true
    ; true ->
        % item 124
        false
    end
.

is_operator(Char) ->
    % item 72
    if ((((((Char =:= $+) orelse (Char =:= $-)) orelse (Char =:= $%)) orelse (Char =:= $*)) orelse (Char =:= $/)) orelse (Char =:= $()) orelse (Char =:= $)) -> 
        % item 78
        true
    ; true ->
        % item 71
        false
    end
.

is_space(Char) ->
    % item 59
    if (((Char =:= 9) orelse (Char =:= 32)) orelse (Char =:= 10)) orelse (Char =:= 13) -> 
        % item 65
        true
    ; true ->
        % item 58
        false
    end
.

lex(Text) ->
    % item 13
    if Text =:= [] -> 
        % item 12
        {ok, []}
    ; true ->
        % item 16
        if is_list(Text) -> 
            % item 18
            lex_kernel(Text, [], [], fun lex_idle/4)
        ; true ->
            % item 6
            {error, "List expected"}
        end
    end
.

lex_finish(Tokens, Buffer) ->
    % item 281
    if Buffer =:= [] -> 
        % item 282
        Completed = Tokens
    ; true ->
        % item 211
        Completed = add_token(Buffer, Tokens)
    end,
    % item 210
    ResultTokens = lists:reverse(Completed),
    {ok, ResultTokens}
.

lex_id(Current, Rest, Tokens, Buffer) ->
    % item 269
    NextState = fun lex_id/4,
    % item 271
    lex_id_number(
    	Current,
    	Rest,
    	Tokens,
    	Buffer,
    	NextState
    )
.

lex_id_number(Current, Rest, Tokens, Buffer, NextState) ->
    % item 285
    if Current =:= $. -> 
        % item 189
        {TokenType, OldText} = Buffer,
        NewText = OldText ++ [Current],
        % item 176
        NewBuffer = {TokenType, NewText},
        % item 163
        lex_kernel(Rest, Tokens, NewBuffer, NextState)
    ; true ->
        % item 162
        Digit = is_digit(Current),
        % item 161
        if Digit -> 
            % item 189
            {TokenType, OldText} = Buffer,
            NewText = OldText ++ [Current],
            % item 176
            NewBuffer = {TokenType, NewText},
            % item 163
            lex_kernel(Rest, Tokens, NewBuffer, NextState)
        ; true ->
            % item 166
            Char = is_char(Current),
            % item 165
            if Char -> 
                % item 189
                {TokenType, OldText} = Buffer,
                NewText = OldText ++ [Current],
                % item 176
                NewBuffer = {TokenType, NewText},
                % item 163
                lex_kernel(Rest, Tokens, NewBuffer, NextState)
            ; true ->
                % item 172
                Space = is_space(Current),
                % item 171
                if Space -> 
                    % item 237
                    process_space(Rest, Tokens, Buffer)
                ; true ->
                    % item 170
                    Op = is_operator(Current),
                    % item 169
                    if Op -> 
                        % item 238
                        process_operator(Current, Rest, Tokens, Buffer)
                    ; true ->
                        % item 187
                        unexpected_char(Current)
                    end
                end
            end
        end
    end
.

lex_idle(Current, Rest, Tokens, _Buffer) ->
    % item 126
    Digit = is_digit(Current),
    % item 125
    if Digit -> 
        % item 143
        NewBuffer = {number, [Current]},
        % item 142
        NextState = fun lex_number/4,
        % item 127
        lex_kernel(Rest, Tokens, NewBuffer, NextState)
    ; true ->
        % item 130
        Char = is_char(Current),
        % item 129
        if Char -> 
            % item 141
            NewBuffer = {id, [Current]},
            % item 144
            NextState = fun lex_id/4,
            % item 127
            lex_kernel(Rest, Tokens, NewBuffer, NextState)
        ; true ->
            % item 138
            Space = is_space(Current),
            % item 137
            if Space -> 
                % item 152
                NewBuffer = {id, []},
                % item 293
                process_space(Rest, Tokens, NewBuffer)
            ; true ->
                % item 135
                Op = is_operator(Current),
                % item 134
                if Op -> 
                    % item 295
                    NewBuffer = {id, []},
                    % item 294
                    process_operator(Current, Rest, Tokens, NewBuffer)
                ; true ->
                    % item 765
                    unexpected_char(Current)
                end
            end
        end
    end
.

lex_kernel(Text, Tokens, Buffer, StateFun) ->
    % item 197
    if Text =:= [] -> 
        % item 200
        lex_finish(Tokens, Buffer)
    ; true ->
        % item 196
        [Current | Rest] = Text,
        % item 201
        if is_number(Current) -> 
            % item 204
            StateFun(Current, Rest, Tokens, Buffer)
        ; true ->
            % item 203
            {error, "Character expected"}
        end
    end
.

lex_number(Current, Rest, Tokens, Buffer) ->
    % item 276
    NextState = fun lex_number/4,
    % item 278
    lex_id_number(
    	Current,
    	Rest,
    	Tokens,
    	Buffer,
    	NextState
    )
.

make_functions() ->
    % item 742
    Items = [
    {"sin",   fun math:sin/1},
    {"cos",   fun math:cos/1},
    {"tan",   fun math:tan/1},
    {"asin",  fun math:asin/1},
    {"acos",  fun math:acos/1},
    {"atan",  fun math:atan/1},
    {"exp",   fun math:exp/1},
    {"log",   fun math:log/1},
    {"log10", fun math:log10/1},
    {"sqrt",  fun math:sqrt/1}
    ],
    % item 743
    dict:from_list(Items)
.

make_operations() ->
    % item 748
    Items = [
    {function, fun calc_function/2},
    {id,       fun calc_id/2},
    {mod,      fun calc_mod/2},
    {number,   fun calc_number/2},
    {prod,     fun calc_prod/2},
    {sum,      fun calc_sum/2}
    ],
    % item 749
    dict:from_list(Items)
.

make_variables() ->
    % item 736
    Items = [
    	{"e",  2.71828182845904523536},
    	{"pi", 3.14159265358979323846}
    ],
    % item 737
    dict:from_list(Items)
.

modulo(Tokens) ->
    % item 465
    {FirstOperand, RestTokens} = product(Tokens),
    % item 434
    if RestTokens =:= [] -> 
        % item 470
        {FirstOperand, RestTokens}
    ; true ->
        % item 433
        [Op | RestTokens2] = RestTokens,
        % item 466
        OpText = token_text(Op),
        % item 467
        if OpText =:= "%" -> 
            % item 469
            {SecondOperand, RestTokens3} = product(RestTokens2),
            % item 441
            Result = [mod, FirstOperand, SecondOperand],
            % item 442
            {Result, RestTokens3}
        ; true ->
            % item 470
            {FirstOperand, RestTokens}
        end
    end
.

parse(Text) ->
    % item 301
    {LexSuccess, Tokens} = lex(Text),
    % item 302
    if LexSuccess =:= ok -> 
        % item 292
        try
        	Tree = parse_kernel(Tokens),
        	{ok, Tree}
        catch
        	Message ->
        		{error, Message}
        end
    ; true ->
        % item 305
        {error, Tokens}
    end
.

parse_kernel(Tokens) ->
    % item 589
    if Tokens =:= [] -> 
        % item 591
        []
    ; true ->
        % item 306
        {Result, Remaining} = sum(Tokens),
        % item 448
        if Remaining =:= [] -> 
            % item 444
            Result
        ; true ->
            % item 445
            throw("Unexpected symbols at the end")
        end
    end
.

process_operator(Current, Rest, Tokens, Buffer) ->
    % item 236
    Tokens2 = add_token(Buffer, Tokens),
    NewBuffer = {op, [Current]},
    NewTokens = [NewBuffer | Tokens2],
    % item 234
    lex_kernel(Rest, NewTokens, [], fun lex_idle/4)
.

process_space(Text, Tokens, Buffer) ->
    % item 280
    Tokens2 = add_token(Buffer, Tokens),
    % item 228
    lex_kernel(Text, Tokens2, [], fun lex_idle/4)
.

prod_item(Element, Acc) ->
    % item 661
    {OldValue, Context} = Acc,
    % item 662
    {Sign, Operand} = Element,
    % item 669
    OperandValue = calculate_kernel(Operand, Context),
    % item 664
    if Sign =:= mul -> 
        % item 663
        Value = OldValue * OperandValue
    ; true ->
        % item 667
        Value = OldValue / OperandValue
    end,
    % item 668
    {Value, Context}
.

product(Tokens) ->
    % item 488
    {FirstOperand, RestTokens} = term(Tokens),
    % item 476
    {Operands, Remaining} = product_loop([], RestTokens),
    % item 477
    if Operands =:= [] -> 
        % item 500
        {FirstOperand, RestTokens}
    ; true ->
        % item 493
        First = {mul, FirstOperand},
        % item 496
        Result = [prod, First | Operands],
        % item 499
        {Result, Remaining}
    end
.

product_loop(Operands, Tokens) ->
    % item 506
    if Tokens =:= [] -> 
        % item 512
        {Operands, Tokens}
    ; true ->
        % item 507
        [First | Rest] = Tokens,
        % item 508
        FirstText = token_text(First),
        % item 5130001
        if FirstText =:= "*" -> 
            % item 522
            Sign = mul,
            % item 510
            if Rest =:= [] -> 
                % item 528
                throw("Expression expected after " ++ FirstText)
            ; true ->
                % item 524
                {Operand, Remaining} = term(Rest),
                % item 525
                SignedOperand = {Sign, Operand},
                % item 526
                NewOperands = Operands ++ [SignedOperand],
                % item 527
                product_loop(NewOperands, Remaining)
            end
        ; true ->
            % item 5130002
            if FirstText =:= "/" -> 
                % item 521
                Sign = divide,
                % item 510
                if Rest =:= [] -> 
                    % item 528
                    throw("Expression expected after " ++ FirstText)
                ; true ->
                    % item 524
                    {Operand, Remaining} = term(Rest),
                    % item 525
                    SignedOperand = {Sign, Operand},
                    % item 526
                    NewOperands = Operands ++ [SignedOperand],
                    % item 527
                    product_loop(NewOperands, Remaining)
                end
            ; true ->
                % item 512
                {Operands, Tokens}
            end
        end
    end
.

sum(Tokens) ->
    % item 368
    if Tokens =:= [] -> 
        % item 371
        throw("Expression expected")
    ; true ->
        % item 386
        [First | Rest] = Tokens,
        % item 387
        FirstText = token_text(First),
        % item 394
        if FirstText =:= "-" -> 
            % item 396
            if Rest =:= [] -> 
                % item 371
                throw("Expression expected")
            ; true ->
                % item 398
                FirstSign = minus,
                % item 399
                RestTokens = Rest,
                % item 402
                {FirstOperand, RestTokens2} = modulo(RestTokens),
                FirstOperandSigned = {FirstSign, FirstOperand},
                % item 367
                {Operands, Remaining} = sum_loop([], RestTokens2),
                % item 372
                if (length(Operands) =:= 0) andalso (FirstSign =:= plus) -> 
                    % item 377
                    Result = FirstOperand
                ; true ->
                    % item 379
                    Result = [sum, FirstOperandSigned | Operands]
                end,
                % item 378
                {Result, Remaining}
            end
        ; true ->
            % item 397
            FirstSign = plus,
            % item 400
            RestTokens = Tokens,
            % item 402
            {FirstOperand, RestTokens2} = modulo(RestTokens),
            FirstOperandSigned = {FirstSign, FirstOperand},
            % item 367
            {Operands, Remaining} = sum_loop([], RestTokens2),
            % item 372
            if (length(Operands) =:= 0) andalso (FirstSign =:= plus) -> 
                % item 377
                Result = FirstOperand
            ; true ->
                % item 379
                Result = [sum, FirstOperandSigned | Operands]
            end,
            % item 378
            {Result, Remaining}
        end
    end
.

sum_item(Element, Acc) ->
    % item 680
    {OldValue, Context} = Acc,
    % item 681
    {Sign, Operand} = Element,
    % item 688
    OperandValue = calculate_kernel(Operand, Context),
    % item 683
    if Sign =:= plus -> 
        % item 682
        Value = OldValue + OperandValue
    ; true ->
        % item 686
        Value = OldValue - OperandValue
    end,
    % item 687
    {Value, Context}
.

sum_loop(Operands, Tokens) ->
    % item 403
    if Tokens =:= [] -> 
        % item 410
        {Operands, Tokens}
    ; true ->
        % item 404
        [First | Rest] = Tokens,
        % item 405
        FirstText = token_text(First),
        % item 4110001
        if FirstText =:= "-" -> 
            % item 420
            Sign = minus,
            % item 408
            if Rest =:= [] -> 
                % item 426
                throw("Expression expected after " ++ FirstText)
            ; true ->
                % item 422
                {Operand, Remaining} = modulo(Rest),
                % item 423
                SignedOperand = {Sign, Operand},
                % item 424
                NewOperands = Operands ++ [SignedOperand],
                % item 425
                sum_loop(NewOperands, Remaining)
            end
        ; true ->
            % item 4110002
            if FirstText =:= "+" -> 
                % item 419
                Sign = plus,
                % item 408
                if Rest =:= [] -> 
                    % item 426
                    throw("Expression expected after " ++ FirstText)
                ; true ->
                    % item 422
                    {Operand, Remaining} = modulo(Rest),
                    % item 423
                    SignedOperand = {Sign, Operand},
                    % item 424
                    NewOperands = Operands ++ [SignedOperand],
                    % item 425
                    sum_loop(NewOperands, Remaining)
                end
            ; true ->
                % item 410
                {Operands, Tokens}
            end
        end
    end
.

term(Tokens) ->
    % item 455
    if Tokens =:= [] -> 
        % item 458
        throw("id, number of function expected")
    ; true ->
        % item 459
        [First | Rest] = Tokens,
        % item 454
        {Type, Text} = First,
        % item 5360001
        if Type =:= number -> 
            % item 462
            Result = [number, Text],
            % item 463
            {Result, Rest}
        ; true ->
            % item 5360002
            if Type =:= id -> 
                % item 551
                if Rest =:= [] -> 
                    % item 555
                    Result = [id, Text],
                    % item 556
                    {Result, Rest}
                ; true ->
                    % item 552
                    [Second | AfterSecond] = Rest,
                    % item 557
                    SecondText = token_text(Second),
                    % item 558
                    if SecondText =:= "(" -> 
                        % item 562
                        {Operand, AfterOp} = sum(AfterSecond),
                        % item 564
                        if AfterOp =:= [] -> 
                            % item 578
                            throw(") expected")
                        ; true ->
                            % item 563
                            [Closing | AfterFun] = AfterOp,
                            % item 565
                            ClosingText = token_text(Closing),
                            % item 566
                            if ClosingText =:= ")" -> 
                                % item 573
                                Result = [function, Text, Operand],
                                % item 574
                                {Result, AfterFun}
                            ; true ->
                                % item 578
                                throw(") expected")
                            end
                        end
                    ; true ->
                        % item 555
                        Result = [id, Text],
                        % item 556
                        {Result, Rest}
                    end
                end
            ; true ->
                % item 550
                if Text =:= "(" -> 
                    % item 579
                    {Operand, AfterOp} = sum(Rest),
                    % item 581
                    if AfterOp =:= [] -> 
                        % item 588
                        throw(") expected")
                    ; true ->
                        % item 580
                        [Closing | AfterBlock] = AfterOp,
                        % item 582
                        ClosingText = token_text(Closing),
                        % item 583
                        if ClosingText =:= ")" -> 
                            % item 587
                            {Operand, AfterBlock}
                        ; true ->
                            % item 588
                            throw(") expected")
                        end
                    end
                ; true ->
                    % item 458
                    throw("id, number of function expected")
                end
            end
        end
    end
.

token_text(Token) ->
    % item 393
    element(2, Token)
.

unexpected_char(Char) ->
    % item 763
    Message = io_lib:fwrite("Unexpected char: ~p", [Char]),
    % item 764
    {error, lists:flatten(Message)}
.


