-module(parser_utest).
-include_lib("eunit/include/eunit.hrl").
%-include(parser).

approx_equal(Left, Right) ->
	Diff = Left - Right,
	abs(Diff) < 0.0000001.

calc(Expected, Expression) ->
	{ok, Value} = parser:calculate(Expression),
	?assert(approx_equal(Expected, Value)).

parser_test() ->
	?assertEqual({ok, []}, parser:lex("")),
	?assertEqual({ok, []}, parser:lex(" ")),
	?assertEqual({ok, [{id, "hi"}]}, parser:lex("hi")),
	?assertEqual({ok, [{id, "hi"}, {id, "there"}]}, parser:lex("hi  there ")),
	?assertEqual({ok, [{number, "2"}, {op, "+"}, {number, "3"}]}, parser:lex("2+3")),
	?assertEqual({ok,[
		{op,"-"},
		{number,"2"},
		{op,"+"},
		{number,"10"},
		{op,"/"},
		{id,"sin"},
		{op,"("},
		{op,"("},
		{id,"m"},
		{op,"-"},
		{number,"10"},
		{op,")"},
		{op,"*"},
		{number,"2.001"},
		{op,"+"},
		{number,"2"},
		{op,"%"},
		{number,"3"},
		{op,")"},
		{op,"-"},
		{number,"0x34AF00"}]},
		parser:lex(" - 2 +10/sin((m - 10)* 2.001 + 2 % 3) - 0x34AF00")),
	?assertEqual({ok, []}, parser:lex(" 	")),
	?assertEqual({ok,[number,"10"]}, parser:parse("10")),
	?assertEqual({ok,[id,"PI"]}, parser:parse("PI")),
	?assertEqual({ok,[sum,{plus,[id,"PI"]},{plus,[number,"2"]}]}, parser:parse("PI + 2")),
	?assertEqual(
		{ok,[sum,
			{minus,[prod,
				 {mul,[sum,{plus,[number,"2"]},{minus,[number,"3"]}]},
				 {mul,[number,"2"]}]},
			{minus,[id,"PI"]},
			{plus,[number,"2"]}]}, parser:parse("-(2 - 3) * 2 - PI + 2")),
	?assertEqual(
		{ok,[sum,
			{minus,[prod,
				 {mul,[sum,{plus,[number,"2"]},{minus,[number,"3"]}]},
				 {mul,[number,"2"]}]},
			{minus,[id,"PI"]},
			{plus,[mod,
				[prod,{mul,[number,"2"]},{divide,[number,"3"]}],
				[prod,{mul,[number,"2.3"]},{mul,[number,"4"]}]]}]}, 
				parser:parse("-(2 - 3) * 2 - PI + 2/3 % 2.3 * 4")),
	?assertEqual(
		{ok,[sum,
			{minus,[prod,
				 {mul,[sum,{plus,[number,"2"]},{minus,[number,"3"]}]},
				 {mul,[number,"2"]},
				 {divide,[number,"10"]}]},
			{plus,[number,"20"]},
			{minus,[sum,
				 {plus,[number,"30"]},
				 {plus,[function,"sin",[number,"0"]]}]}]}, 
		parser:parse("-(2 - 3) * 2 /10 + 20 - (30 + sin(0))")),
	?assertEqual(
		{ok,[prod,
			{mul,[sum,{plus,[number,"2"]},{minus,[number,"3"]}]},
			{divide,[sum,{minus,[number,"2"]}]},
			{mul,[sum,{plus,[number,"4"]},{plus,[number,"5"]}]}]}, 
		parser:parse("(((2-3)/(-2)*(4+5)))")),

	calc(0, ""),
	calc(0, "  "),
	calc(0, " 0 "),
	calc(75, "75"),
	calc(75.1, "75.10"),
	calc(5, "2 + 3"),
	calc(14, "2 + 3 * 4"),
	calc(100, "-8 / 2 * 3 + 3 * 4 + 100"),
	calc(1101, "1000 + (1 + (8)) % 2 + 100"),
	calc(-399, "sin(pi / 2) - 400 * (log(e))"),
	calc(1, "sin(pi / 4 + pi / 4)"),
	calc(-102.66666666666667, "(-(23 + 45 % (7 * 2) - 3 / (8+1)) * 4)"),
     
	?assertEqual({ok, []}, parser:parse("")).